//
// smsexporterappui.cpp
//
// Copyright 2004 by the authors.  All rights reserved.
//
// Authors: Tero Hasu <tero.hasu@hut.fi>
//

#include <aknnotedialog.h>
#include <aknnotewrappers.h>
#include <avkon.hrh>
#include "smsexporter.hrh"
#include <maebian_smsexporter.rsg>
#include "smsexporterappui.h"
#include "smsexportercontainer.h"

#define KEnableSkinFlag 0x1000
#define KLayoutAwareFlag 0x08

TInt ProgressCallBack(TAny* aArg)
{
  CSmsexporterAppUi* p = static_cast<CSmsexporterAppUi*>(aArg);
  // best effort to report progress
  TRAPD(error, p->ProgressedL());
  return error; // return value not actually used at present
}

void CSmsexporterAppUi::ConstructL()
{
#ifdef __SERIES60_30__
  BaseConstructL(EAknEnableSkin);
#else
  BaseConstructL(KEnableSkinFlag | KLayoutAwareFlag);
#endif

  iCallBack.iFunction = ProgressCallBack;
  iCallBack.iPtr = this;

  iEngine = CExporterEngine::NewL(CEikonEnv::Static()->FsSession());

  iAppContainer = new (ELeave) CSmsexporterContainer;
  iAppContainer->SetMopParent( this );
  iAppContainer->ConstructL( ClientRect() );
  AddToStackL( iAppContainer );

  UpdateStatusL();
}


// we do not call this directly, only the callback does
void CSmsexporterAppUi::ProgressedL()
{
  if (iEngine->iComplete) {
    // the operation completed;
    // use a note dialog to tell the user
    TInt status = iEngine->iStatus.Int();


    if (status == KErrNone) {
      _LIT(KDoneText,"done");
      CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
      informationNote->ExecuteLD(KDoneText);
    } else {
      _LIT(KErrorText,"error");
      CAknErrorNote* errorNote = new (ELeave) CAknErrorNote;
      errorNote->ExecuteLD(KErrorText);
    }



	/* dynamic text is not straightforward, cannot do this
    CAknNoteDialog* note;
    TBuf<50> noteText;
    if (status == KErrNone) {
      noteText.Copy(_L("done"));
      note = new (ELeave) CAknInformationNote;
    } else {
      noteText.Copy(_L("error "));
      noteText.AppendNum(status);
      note = new (ELeave) CAknErrorNote;
    }
    // the system will free the note, I believe
    note->ExecuteLD(noteText);
	*/


  }

  UpdateStatusL();
}

void CSmsexporterAppUi::UpdateStatusL()
{
  TBuf<50> statusText;
  TBuf<50> progressText;

  if (iEngine->iComplete) {
    statusText.Copy(_L("idle"));
    progressText.Num(iEngine->iList->Count());
    progressText.Append(_L(" messages"));
  } else {
    if (iEngine->iDeleting) {
      statusText.Copy(_L("deleting"));
    } else {
      statusText.Copy(_L("exporting"));
    }
    progressText.Num(iEngine->iNextIndex);
    progressText.Append(_L(" "));
    progressText.AppendNum(iEngine->iList->Count());
  }

  iAppContainer->SetStatusL(statusText, progressText);
}



CSmsexporterAppUi::~CSmsexporterAppUi()
{
  delete iEngine;

  if (iAppContainer)
    {
      RemoveFromStack(iAppContainer);
      delete iAppContainer;
    }
}

void CSmsexporterAppUi::DynInitMenuPaneL(TInt /*aResourceId*/, 
					 CEikMenuPane* /*aMenuPane*/)
{
}

TKeyResponse CSmsexporterAppUi::HandleKeyEventL
(const TKeyEvent& /*aKeyEvent*/, TEventCode /*aType*/)
{
  return EKeyWasNotConsumed;
}

void CSmsexporterAppUi::HandleCommandL(TInt aCommand)
{
  switch (aCommand)
    {
    case EAknSoftkeyBack:
    case EAknSoftkeyExit:
    case EEikCmdExit:
      {
	Exit();
	break;
      }
    case EsmsexporterCmdExport:
      {
	if (iEngine->iComplete)
	  iEngine->ExportMessagesL(iCallBack);
	break;
      }
    case EsmsexporterCmdDelete:
      {
	if (iEngine->iComplete)
	  iEngine->DeleteMessagesL(iCallBack);
	break;
      }
    default:
      break;
    }
}
